﻿using MvvmHelpers;
using SMM.Automation;
using SMM.Command;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Management.Automation;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace MyLLDP
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            DataContext = new MainWindowViewModel(this);
        }
    }


    public class MainWindowViewModel : BaseViewModel
    {
        private readonly MainWindow _parent;
        public MainWindowViewModel(MainWindow parent)
        {
            _parent = parent;

            ScriptRunner = new SimpleScriptRunner();
            ScriptRunner.ScriptFragments.Add(ScriptBlock.Create(Properties.Resources.script1));
            ScriptRunner.ScriptFragments.Add(ScriptBlock.Create(Properties.Resources.script2));

            ScriptRunner.PropertyChanged += ScriptRunner_PropertyChanged;

            ScanEnabled = true;
        }


        private void ScriptRunner_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Results")
            {
                OnPropertyChanged(nameof(Results));
            }
        }


        private string _results;
        public string Results { get {
                _results = String.Join(Environment.NewLine, ScriptRunner.Results.Select(x => x.ToString()));
                return _results;
            }
        }


        private bool _enabled = false;
        public bool ScanEnabled { get => _enabled; set => SetProperty(ref _enabled, value); }

        private bool _ShowClearButton = false;
        public bool ShowClearButton { get => _ShowClearButton; set => SetProperty(ref _ShowClearButton, value); }



        private SimpleScriptRunner _scriptRunner;
        public SimpleScriptRunner ScriptRunner { get => _scriptRunner; set => SetProperty(ref _scriptRunner, value); }


        private RelayCommand _startCap;
        public ICommand StartCapture {
            get {
                if (_startCap == null)
                {
                    _startCap = new RelayCommand(
                        () => ScriptRunner?.Run());
                }
                return _startCap;
            }
        }

        private RelayCommand _toggleShowClear;
        public ICommand ToggleShowClearButton {
            get {
                if (_toggleShowClear == null)
                {
                    _toggleShowClear = new RelayCommand(
                        () => this.ShowClearButton = true);
                }
                return _toggleShowClear;
            }
        }

        
        private RelayCommand _ClearNetSessions;
        public ICommand ClearNetSessions {
            get {
                if (_ClearNetSessions == null)
                {
                    _ClearNetSessions = new RelayCommand(
                        async () => {
                            var runner = new SimpleScriptRunner(Properties.Resources.ClearNetSessions);
                            await runner.Run();
                        });
                }
                return _ClearNetSessions;
            }
        }

    }
}
